function [ret,phan]=Aqual2_findSphereSmallSpheres(phan,verbose)

if ~exist('verbose','var')
    verbose=0;
end
filename=phan.filename; % used only for error handling

maxBBoxErrors=4;

propList={ 'Area', 'Centroid', 'BoundingBox','SubarrayIdx','PixelList','PixelIdxList'};
if ~isfield(phan,'voxTheoryPositions')
    estring=sprintf('bug: voxTheoryPositions has not been set');
    Aqual2_writeFailureLog(phan.outputFilePrefix,estring);
    ret=99; % this should never happen!
    return;
end
voxSize=phan.hdr.dime.pixdim(2:4);
sphereRadius=5.0;
searchSize=15.0; % that's a half-width --
targetVolume=4/3*pi*(sphereRadius^3) /phan.voxelVolume;
%phan.foundSphere=zeros(length(phan.voxTheoryPositions),1);
% foundSphere is trinary logic:  -1(didn't find it), 0 (haven't looked),
% 1(found it)
templateRadius=sphereRadius+3;
[x,y,z]=meshgrid(-templateRadius:voxSize(2):templateRadius,...
    -templateRadius:voxSize(1):templateRadius,...
    -templateRadius:voxSize(3):templateRadius);
T = sqrt ( x.^2 + y.^2 + z.^2 ) < sphereRadius;
 
sphereTemplateSize = size ( T );
sphereTemplate=T;

nhood=ones(3,3,3);
se=strel(nhood);
clear nhood;
shellTemplate=imdilate(sphereTemplate,se);
shellTemplate=imdilate(shellTemplate,se); % wrap 2 pixel border around the sphere
%backgroundTemplate=imdilate(shellTemplate,se);
%backgroundTemplate=imdilate(backgroundTemplate,se);
%backgroundTemplate=backgroundTemplate-shellTemplate;

shellTemplate= (shellTemplate==0);  % flip it to turn on area outside sphere+border;
shellTemplate=shellTemplate+sphereTemplate;  % put the contrast sphere back in, leaving a dark ring
%
%for target=4:length(phan.voxTheoryPositions)-4


bboxErrorCounter=0;
for whichTarget=1:length(phan.searchOrder)
    target=phan.searchOrder(whichTarget);
    if verbose>5
        tic
    end
    % we have to fake prop  -- flip the xy  so it looks like something that
    % came out of regionprops  -- this allows us to re-use
    % getClippedSubRegion
    if verbose>3 
        disp(sprintf('searching for sphere number %d (%d of 158)',target,whichTarget));
    end
    prop=struct('Centroid',...
        [phan.voxTheoryPositions(target,2),phan.voxTheoryPositions(target,1),phan.voxTheoryPositions(target,3)]);

    [regionImg,subIdx]=getClippedSubRegion(phan,searchSize,prop);
    regionVolume=length(regionImg(:));
    % find the 3D correlation image (takes about 200-400ms  P43GHz)
     try
    cshellimg=normxcorr3(shellTemplate,regionImg,'same');
     catch
            ret=2;
            disp('failed to run normxcorr3');
             estring=sprintf('failed running 3D correlation for sphere target %d -- possibly insufficient FOV',target);
             Aqual2_writeFailureLog(phan.outputFilePrefix,estring,phan);
             return;  
     end
    csphereimg=normxcorr3(sphereTemplate,regionImg,'same');
    cimg=cshellimg.*csphereimg;
    
    % ccMask get's plunked down on top of the correlation image to help make
    % sure we only find the sphere we are looking for  --- it's sized against
    % the minSphereSeparation (which is in mm)
 

%[x,y,z]=meshgrid(-phan.minSphereSeparation:voxSize(2):phan.minSphereSeparation,...
 %   -phan.minSphereSeparation:voxSize(1):phan.minSphereSeparation,...
  %  -phan.minSphereSeparation:voxSize(3):phan.minSphereSeparation);
  %  ccMask = sqrt ( x.^2 + y.^2 + z.^2 ) < phan.minSphereSeparation;
    % now we need to cut out a ccMask-sized area in the middle of cimg
    cimgSize=size(cimg);
    cMid=cimgSize/2.0;
    for ii=1:cimgSize(1)
        for jj=1:cimgSize(2)
               for kk=1:cimgSize(3);
                   xx=(cMid(1)-ii)*voxSize(2);
                   yy=(cMid(2)-jj)*voxSize(1);
                   zz=(cMid(3)-kk)*voxSize(3);
                   if (sqrt ( xx.^2 + yy.^2 + zz.^2 ) >phan.minSphereSeparation)
                       cimg(ii,jj,kk)=0.0;
                   end
               end
        end
    end
                     
    
    
    
    cimgMaxInd=find(cimg(:)==max(cimg(:)));
    [cimgMaxSub(1),cimgMaxSub(2),cimgMaxSub(3)]=ind2sub(size(cimg),cimgMaxInd);
    % now use correlation weighted average over 5x5x5 cube centered on the
    if verbose>100
    cimgMaxSub
    end
    % do the dumb thing and do a triple nested loop; more matlab-ly
    % approaches I've tried are slower.
    cimgPos=[0,0,0];
    cimgSum=0;
    nInSum=0;
    window=1;  % 3x3x3 cube of voters  (max)
    [imax,jmax,kmax]=size(regionImg);
    try
        ilow=cimgMaxSub(1)-window;
        if(ilow<1)
            ilow=1;
        end

        ihigh=cimgMaxSub(1)+window;
        if(ihigh>imax)
            ihigh=imax;
        end

        jlow=cimgMaxSub(2)-window;
        if(jlow<1)
            jlow=1;
        end

        jhigh=cimgMaxSub(2)+window;
        if(jhigh>jmax)
            jhigh=jmax;
        end

        klow=cimgMaxSub(3)-window;
        if(klow<1)
            klow=1;
        end

        khigh=cimgMaxSub(3)+window;
        if(khigh>kmax)
            khigh=kmax;
        end


        for ii=ilow:ihigh
            for jj=jlow:jhigh
                for kk=klow:khigh
                    if((cshellimg(ii,jj,kk)>0) &&  (csphereimg(ii,jj,kk)>0))
                        nInSum=nInSum+1;
                        cimgSum=cimgSum+cimg(ii,jj,kk);
                        cimgPos(1)=cimgPos(1)+ii*cimg(ii,jj,kk);
                        cimgPos(2)=cimgPos(2)+jj*cimg(ii,jj,kk);
                        cimgPos(3)=cimgPos(3)+kk*cimg(ii,jj,kk);
                    end
                end
            end
        end
    catch
        disp('bad bad oob bug in findSphereSmallSpheres');
    end
    if nInSum==0
            % wow no overlap between positive correlations of the shell and
            % sphere image correlations  -- can happen on very low relative
            % sphere contrast with relatively high now levels 
            % we are cowards --- abort 
           disp(sprintf('Note: cross correlations are inconsistent --aborting search for sphere %d',target));
           found=0;
    else
            
        
        phan.nCorrPosElements(target)=nInSum;
        cimgPos=cimgPos./cimgSum;
        phan.corrPositions(target,1:3)=cimgPos+[subIdx{1}(1),subIdx{2}(1),subIdx{3}(1)]-[1,1,1];
        cimgSize=size(cimg);
        %   imshow(cimg(:,:,floor(cimgSize(3)/2)));
        tmp=[phan.corrPositions(target,1:3),1];
        tmp=phan.RAS*tmp';
        phan.RAScorrPositions(target,1:3)=tmp(1:3)';
        cfloor=floor(cimgPos);
        if verbose>100
            cfloor
        end
        cindex=sub2ind(size(regionImg),cfloor(1),cfloor(2),cfloor(3));
        % we'll come back to the correlation image later as a confirmatory tool
        % first we'll try to use just intensities to get a segmentation

        % histogram the intensities until we find an upper limit in which we
        % have 95% of the targetVolume on.  Then we'll walk the threshold down
        % until we either get a cluster of about the right volume of it blows up
        [n,x]=hist(regionImg(:),100);
        csum=cumsum(n);
        csum=find(csum<(regionVolume-0.8*targetVolume));
        thresh=x(csum(end));
        props=[];
        found=0;
        while (found==0)&&(thresh>0.10*x(end))
            if verbose >3
                thresh
            end
            timg=regionImg>thresh;
            limg=bwlabeln(timg);
            allprops=regionprops(limg,propList);
            % now sort through allprops to see if there is a cluster that is about
            % the right size
            defect=([allprops.Area] - targetVolume)./targetVolume;
            dlow=defect>-0.25;
            dhigh=defect<0.1;
            dall=dlow.*dhigh;
            inWindow=find(dall);
            if(length(inWindow)==0)
                thresh=thresh*0.99;
            else
                % we have at least one cluster of the right volume
                % is it consistent with the cimgPos?
                whichLabel=limg(cindex);
                if(whichLabel>0)
                    % is inWindow(whichLabel) set ?  (is cluster the right one);
                    if(dall(whichLabel)==1)
                        % then we found the cluster
                        found=1;
                        props=allprops(whichLabel);
                    else
                        thresh=thresh*0.99;
                    end
                else
                    % whichLabel==0 -- labels and corr center are inconsistent
                    % may be a bright object somewhere else in the fov -- lower
                    % threshold and try again.
                    thresh=thresh*0.99;
                end
            end
        end
    end

    if(found==0)
        estring=sprintf('target %d not found',target);
        disp(estring);
        phan.foundSphere(target)=0;
    else
        limg=imfill(limg,26,'holes');
        limg=bwlabeln(limg);
        whichLabel=limg(cindex);
        if(whichLabel==0)
            disp('error --- findSphereSmallSpheres -- this cannot happen');
            ret=1;
            return;
        end
        % save the information about the cluster
        limg=(limg==whichLabel);
        if verbose>5
            toc
        end
        nhood=ones(3,3,3);
        se=strel(nhood);
        clear nhood;
        tmpMask=imdilate(limg,se);
        tmpMask=imdilate(tmpMask,se);
        tmpbkgd=imdilate(tmpMask,se);
        tmpbkgd=imdilate(tmpbkgd,se);
        tmpbkgd=tmpbkgd-tmpMask;
        phan.backgroundIntensityMedian(target)=median(regionImg(find(tmpbkgd)));
        if verbose>5
            toc
        end
        sz=size(regionImg);
        bboxprop=regionprops(bwlabeln(limg),'boundingbox');
        %         for ii=1:sz(1)
        %             for jj=1:sz(2)
        %                 for kk=1:sz(3)
        try
            for ii=floor(bboxprop.BoundingBox(2)):ceil(bboxprop.BoundingBox(2)+bboxprop.BoundingBox(2+3))
                for jj=floor(bboxprop.BoundingBox(1)):ceil(bboxprop.BoundingBox(1)+bboxprop.BoundingBox(1+3))
                    for kk=floor(bboxprop.BoundingBox(3)):ceil(bboxprop.BoundingBox(3)+bboxprop.BoundingBox(3+3))
                        if(phan.masterLabelImg(ii+subIdx{1}(1)-1,jj+subIdx{2}(1)-1,kk+subIdx{3}(1)-1)==0)...
                                phan.masterLabelImg(ii+subIdx{1}(1)-1,jj+subIdx{2}(1)-1,kk+subIdx{3}(1)-1)=...
                                target*limg(ii,jj,kk);
                        end
                    end
                end
            end
        catch
            disp('bbox problems');
             bboxErrorCounter=bboxErrorCounter+1;   
              if bboxErrorCounter>maxBBoxErrors
                    ret=50;
                    return;
              end   
        end
        phan=findObjectPositions(phan,target);
        phan.foundSphere(target)=1;
    end
    % now take all the spheres that have been found and update the theory
    % positions -- because we are doing an explicit search there is no
    % guessing about correspondence
    if verbose>5
        toc
    end
    if sum(phan.foundSphere)>5
        % update the theory positions 
       foundIndex=find(phan.foundSphere);
       %          foundLocs=phan.voxCentroids(foundIndex,:);
       foundLocs=phan.corrPositions(foundIndex,:);
       theoryLocs=phan.voxUnfittedTheoryPositions(foundIndex,:);
       if sum(phan.foundSphere)>5
           OPP = OrthogonalProcrustes ( foundLocs',theoryLocs');
           if det(OPP)>0
               OPPTForm = maketform ( 'affine', OPP' );
               theoryLocs=tformfwd(theoryLocs,OPPTForm);
               % now apply the tform to *all* the voxUntifttedTheoryPositions
               % so we can feed updated search positions forward
               phan.voxTheoryPositions=tformfwd(phan.voxUnfittedTheoryPositions,OPPTForm);
               if sum(phan.foundSphere)>30
                   fitcoeff=FitSpheres(theoryLocs,foundLocs,1);
                   delta=EvalPolyFit(fitcoeff,foundLocs,1);
                   theoryLocs=theoryLocs-delta;
                   % again apply to all
                   delta=EvalPolyFit(fitcoeff,phan.voxTheoryPositions,1);
                   phan.voxTheoryPositions=phan.voxTheoryPositions-delta;
               end
           else
               disp('Warning:  flip found in OPP transform --- ignoring transform for now');
           end
               if verbose>3
                D = theoryLocs-foundLocs;
                if verbose> 10
                    hist(D)
                end
                D = sqrt ( sum ( D'.^2 ) );
                disp ( sprintf ( 'Max distance: %f', max ( D ) ) );
            end
       end
    end
    if verbose>5
        toc
        if verbose>500
            p=phan
            p.img=p.masterLabelImg;
            save_nii(p,'working_seg.nii');
        end
    end
end
ret=0;
%disp('foo');
return;
