function OptTxt = ADNIEvaluateLinearity ( Baseline, Calibration, OptTxt)
    
  Baseline = ReorderSpheres ( Calibration, Baseline );
  Idx = [Baseline.FiducialSphereIndex Baseline.LandmarkSphereIndex];
  Calibration.Spheres = Calibration.Spheres(Idx);
  Baseline.Spheres = Baseline.Spheres(Idx);
  
  OPP = OrthogonalProcrustes ( [Calibration.Spheres.CanonicalCOM], [Baseline.Spheres.CanonicalCOM] );
    
  OPPTForm = maketform ( 'affine', OPP' );

  RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
  BaselinePointsCan = tformfwd ( [Baseline.Spheres.CanonicalCOM]', OPPTForm );
  BaselinePointsRAS = tforminv ( BaselinePointsCan, RASToCanonicalTForm )';

  Parameters = WarpDefaults;
  Parameters.DoWarp = 0;  
  
  % Linear
  Parameters.SplineType = 'affine';
  [NewImage, Results] = WarpPhantom ( [], PatientData(Calibration.Header), Baseline, Calibration, Parameters );
  NewPoints = tforminv ( BaselinePointsRAS', Results.Warp.tdata(2) )';
  RMS = sqrt ( mean ( sum ( ( NewPoints - [Calibration.Spheres.CenterRAS] ).^2 ) ) );
  % disp ( sprintf ( 'Linear RMS: %f', RMS ) );


  % TPS
  if 0
    Parameters.SplineType = 'tps';
    [NewImage, Results] = WarpPhantom ( [], PatientData(Calibration.Header), Baseline, Calibration, Parameters );
    NewPoints = tforminv ( BaselinePointsRAS', Results.Warp.tdata(2) )';
    RMS = sqrt ( mean ( sum ( ( NewPoints - [Calibration.Spheres.CenterRAS] ).^2 ) ) );
    disp ( sprintf ( 'TPS RMS: %f', RMS ) );
  end
  
  
  for order = 1:5
    CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];

    Y = CalibrationPointsRAS';
    X = BaselinePointsRAS';
    % Coefficients = FitSpheres ( CalibrationPointsRAS', BaselinePointsRAS', order );
    Coefficients = FitSpheres ( Y, X, order );
    Displacement = EvalPolyFit ( Coefficients, X, order );
    NewPoints = Y - Displacement;
    
    % NewPoints = EvalPolyFit ( Coefficients, BaselinePointsRAS', order );
    RMS = sqrt ( mean ( sum ( ( NewPoints' - X' ).^2 ) ) );
    disp ( sprintf ( 'Polynomial (%d order) RMS: %f', order, RMS ) );
    if exist ('OptTxt','var') 
        OptTxt=sprintf('%s Polynomial (%d order) RMS: %f\n', OptTxt,order, RMS );
    end 
  end

  
  
return  
% Linear vs. polynomial fits
