


% This is a simple example of how to run the ADNI calibration
% phantom segmentation code.  First set up a helper array that
% contains the names of the DICOM series to process.

S = [];
Baseline = [];
  
S = [{'BaselinePhantom'}, 2];
S = [S; [{'FollowupPhantom'}, 2]];
for ii = 2:5
  S = [S; [{'SNR'}, ii]];
end

Report = sprintf ( '%s\n', ADNIReport );
  
% Process each in turn.
for Sidx = 1:length(S)
  Subject = char(S(Sidx,1));
  Series = cell2mat ( S(Sidx,2) );
  Pattern = sprintf ( 'DICOM/%s/Series%03d/*.dcm', Subject, Series );
  disp ( sprintf ( 'Loading: %s', Pattern ) );
  
  % If the images are not sorted, just LoadDICOMSeries, if they
  % are sorted as Image####.dcm, use LoadDICOMSeriesFast.  This
  % was written because the Matlab DICOM reader is really slow.
  % [Image, Headers] = LoadDICOMSeries ( Pattern );
  [Image, Headers] = LoadDICOMSeriesFast ( Pattern );
  
  % Segment the spheres
  [Spheres LI] = SegmentSpheresADNI ( Headers, Image, [], 1 );
  Spheres.SNR = CalculateSNRFromSphere ( Image, LI == Spheres.SNRSphereIndex );
  
  disp ( sprintf ( 'Subject: %s Series: %d', Subject, Series ) );
  
  if isempty ( Baseline )
    Baseline = Spheres;
  end
  
  % try
    % If the segmentation failed it will show up here when we try
    % to reorder the spheres, hence the try-catch blocks.
    [FollowUp, LI, Mapping] = ReorderSpheres ( Baseline, Spheres, LI );
    Spheres = FollowUp;
    Line = ADNIReport ( Headers, Image, Baseline, FollowUp );
    Report = sprintf ( '%s%s\n', Report, Line );
  % catch 
    % display ( sprintf ( 'Failed: %s Series: %d', Subject, Series ) );
  % end
  
  % Save the image and ObjectMap
  FN = sprintf ( '%s-%d', Subject, Series );
  ADNISaveAnalyze ( FN, Headers, Image, Spheres, LI )
end


% Load the last segmented objectmap to verify that all is the
% same
ObjectMap = LoadAVWObjectMap ( [FN '.obj'] );
LabelMap = ObjectMap.Image;
[Spheres LI] = SegmentSpheresADNI ( Headers, Image, LabelMap > 1, 1 );
Spheres.SNR = CalculateSNRFromSphere ( Image, LI == Spheres.SNRSphereIndex );

disp ( sprintf ( 'Validation of SNR calculation: %f', Spheres.SNR ) );
disp ( sprintf ( 'Validation of Sphere locations (should be 0.0):' ) );
[Spheres] = ReorderSpheres ( Baseline, Spheres );
Line = ADNIReport ( Headers, Image, Baseline, FollowUp );
Report = sprintf ( '%s%s\n', Report, Line );

disp ( 'Final report: \n\n' );
disp ( Report );