function ADNISaveAnalyze ( Filename, Headers, Image, Spheres, LabeledImage )
% function ADNISaveAnalyze ( Filename, Headers, Image, Spheres, LabeledImage )
% Save the image, labeledimage based on the Filename and Spheres
% information
  
  [width height depth] = size ( LabeledImage );
  avw = ConvertDICOMToAVW ( Image, Headers );
  OM = CreateObjectMap ( width, height, depth );
  OM.Image = LabeledImage + 1;
  
  Flip = 0;
  if Flip
    switch lower ( avw.Orientation )
     case { 'transverse' }
      OM = CreateObjectMap ( width, height, depth );
      OM.Image = LabeledImage;
      % Transpose the slices
      yy = size(OM.Image,1):-1:1;
      for ii = 1:size(OM.Image,3)
        t = squeeze(OM.Image(:,:,ii))';
        t = t(:,yy);
        OM.Image(:,:,ii) = t;
      end
      
     case { 'sagittal' }
      OM = CreateObjectMap ( depth, width, height );
      sz = size ( avw.Image );
      T = zeros ( sz(3), sz(2), sz(1) );
      for slice = 1:sz(1)
        T(:,:,slice) = squeeze ( LabeledImage(width-slice+1,:,:) )';
      end
      OM.Image = T;
      clear T;
      
     case { 'coronal' }
      OM = CreateObjectMap ( width, depth, height );
      sz = size ( avw.Image );
      T = zeros ( width, depth, height );
      for slice = 1:height
        T(:,:,slice) = squeeze ( LabeledImage(width:-1:1,slice,:) );
      end
      OM.Image = T;
      clear T;
    end
  end

  for ii=1:length(Spheres.Spheres)
    Name = sprintf ( 'Sphere %d', ii );
    if length ( find ( Spheres.LandmarkSphereIndex == ii ) )
      Name = sprintf ( 'Sphere %d Landmark', ii );
    end
    OM = NewObject ( OM, Name );
  end
  % Transpose the slices
  yy = size(OM.Image,1):-1:1;
  for ii = 1:size(OM.Image,3)
    t = squeeze(OM.Image(:,:,ii))';
    t = t(:,yy);
    OM.Image(:,:,ii) = t;
  end
  FN = Filename;
  
  avw.Orientation = 'Transverse';
  avw.AnalyzeOrient = 0; % Transverse/axial
  avw.Image = Image;
  
  SaveAVW ( [FN '.avw'], avw );
  
  SaveAVWObjectMap ( [FN '.obj'], OM );
