function [Properties, T, Idx] = CalculateCanonicalTransformADNI(Properties)
% CalculateCanonicalTransform - Determine canonical sphere numbers by transforming to regular space and relabeling each sphere
% Requires a Properties structure from ProcessPhantom
% Works with scanner COM, which is in mm and in the RAS coordinate system

  if isfield ( Properties, 'Spheres' )
    Properties = Properties.Spheres;
  end
  
  CenterRAS = [Properties.CenterRAS]';
  
  % Sort and take the three largest
  [Y, idx] = sort ( [Properties.Volume] );
  r = length(idx):-1:1;
  Y = Y(r);
  idx = idx(r);

  % Sidx = [Properties.SNRSphereIndex Properties.Poles];
  if length(idx) == 161
    Sidx = idx(1:3);
  else
    Sidx = idx([1 6 7]);
  end
    
  S1 = Properties(Sidx(1)).CenterRAS;
  S2 = Properties(Sidx(2)).CenterRAS;
  S3 = Properties(Sidx(3)).CenterRAS;

  D = [];
  D(1) = sqrt ( sum ( ( S1 - S2 ).^2 ) );
  D(2) = sqrt ( sum ( ( S2 - S3 ).^2 ) );
  D(3) = sqrt ( sum ( ( S1 - S3 ).^2 ) );
  
  % sort smallest to largest
  [DD, Didx] = sort ( D );
  
  % Find the sphere left out in the D calculations
  Map = [3 1 2];

  Idx = Sidx(Map(Didx));
  SXidx = Idx(1);
  SYidx = Idx(2);
  SOidx = Idx(3);

  % X is the further out sphere
  % Y is the closer sphere
  SO = CenterRAS(SOidx,:);
  SX = CenterRAS(SXidx,:);
  SY = CenterRAS(SYidx,:);

  debug = 0;
  if debug
  disp ( sprintf ( 'SO: %f, %f, %f', SO(1), SO(2), SO(3) ) );
  disp ( sprintf ( 'SX: %f, %f, %f', SX(1), SX(2), SX(3) ) );
  disp ( sprintf ( 'SY: %f, %f, %f', SY(1), SY(2), SY(3) ) );
  end
  % Now we have the Origin, Xhat, Yhat spheres 
  % Define a coordinate system change that maps from the CenterRAS space,
  % to the internal coordinate system defined by the vectors.
  Ttranslate = eye(4);
  Ttranslate(1:3,4) = -SO';
  
  Trotate = eye(4);
  Xhat = ( SX - SO ) ./ norm ( SX - SO );
  Yhat = ( SY - SO ) ./ norm ( SY - SO );
  Zhat = cross ( Xhat, Yhat );
  Yhat = cross ( Zhat, Xhat );
  Xhat = cross ( Yhat, Zhat );

  if debug
  disp ( sprintf ( 'Xhat: %f, %f, %f', Xhat(1), Xhat(2), Xhat(3) ) );
  disp ( sprintf ( 'Yhat: %f, %f, %f', Yhat(1), Yhat(2), Yhat(3) ) );
  disp ( sprintf ( 'Zhat: %f, %f, %f', Zhat(1), Zhat(2), Zhat(3) ) );
  end
  
  Trotate(1:3,1:3) = [Xhat' Yhat' Zhat']';
  if debug
  disp ( 'Trotate' );
  Trotate(1:3,1:3)
  end
  % Tscale = eye(4) ./ norm ( SZ - SO );
  % Tscale(4,4) = 1;
  
  % T is the transformation that transforms CenterRAS points to points in
  % "Phantom" space.
  % Don't want to scale.
  T = Trotate * Ttranslate;
  
  len = size ( CenterRAS, 1 );
  CenterRASPrime = [CenterRAS ones(len,1)]';
  NewCenterRAS = T * CenterRASPrime;
  NewCenterRAS = NewCenterRAS';
  NewCenterRAS = NewCenterRAS(:,1:3);
  
  for ii = 1:len,
    Properties(ii).CanonicalCOM = NewCenterRAS(ii,:)';
  end
  
