function [P, S] = CreateVirtualPhantom ( x, y, z, pradius, sradius, sspacing )

% Create an image of size (x,y,z) 
% containing a phantom of radius pradius with size sradius spheres, spaced
% at sspacing
% Return P, the phantom image, and S the true centers of the
% spheres, in image coordinates
% Spheres are set to 1, background set to 0

  
  P = zeros ( x, y, z );
  center = round ( size ( P ) ./ 2 );
  S = [];
  
  % Back through w/distance transform and threshold
  zrad = (1:z) * 0;
  xidx = (1:x) - center(1);
  yidx = (1:y) - center(2);
  zidx = (1:z) - center(3);
  xx = find ( mod ( xidx, sspacing ) == 0 );
  yy = find ( mod ( yidx, sspacing ) == 0 );
  zz = find ( mod ( zidx, sspacing ) == 0 );
  for k = zz
    % Set radius at this point
    zpos = k - center(3);
    rad = sradius - sqrt ( (zidx - zpos).^2);
    Idx = find ( rad >= 0 );
    % rad(Idx)
    zrad(Idx) = rad(Idx);
    % plot ( zrad )
    % pause
  end

  % We really don't care how these spheres look, so just pop 'em
  % out
  pradius2 = pradius^2;
  for k = zz,
    zpos = center(3) - k;
    for j = yy,
      ypos = center(2) - j;
      for i = xx,
	% Does a sphere fit here?
	xpos = center(1) - i;
        % disp ( sprintf ( 'Checking: %d %d %d', i, j, k ) )
	if ( (xpos^2 + ypos^2 + zpos^2 + sspacing^2) < pradius2 )
          % disp ( sprintf ( 'Drawing' ) )
	  % disp ( sprintf ( 'Drawing at: %d, %d, %d', xpos, ypos, zpos ) )
	  S = [S; i j k];
          rad = sradius - sqrt ( (zidx - (k-center(3))).^2) ;
          Idx = find ( rad >= 0 );
	  for kk = Idx,
	    % Put a point on each slice
            % disp ( sprintf ( '\tPoint at: %d, %d, %d, Radius: %f', i, j, kk, zrad(kk)  ) )
	    P(i,j,kk) = 1;
	  end
	end
      end
    end
  end
 
  % h = waitbar ( 0, 'Creating phantom...' );
  for k = 1:z,
    % Threshold the distance transform
    % disp ( sprintf ( 'Slice: %d/%d', k, z ) )
    % waitbar ( k / z );
    if zrad(k) > 0.0
      P(:,:,k) = bwdist( P(:,:,k) ) <= zrad(k);
    end
    % disp ( sprintf ( 'Radius at: %d is: %f', k, zrad(k) ) )
  end
  % close ( h )