% Baseline for this data is Series 3
% Data name is CalibrationSeries
% Interesting data here is 3:25
load CalibrationExam39

% Baseline for this series is Series 2
% Data name is NonlinearSeries
% Interesting data here is 3-12
load NonlinearSeries


Baseline = CalibrationSeries(3);
CalibrationData = [CalibrationSeries(16) NonlinearSeries(6)];


fid = fopen ( 'Reports/Report-2003-03-12-EvaluateWarp.txt', 'w' );
fprintf ( fid, 'Try different methods of warping our calibration data\n' );
fprintf ( fid, sprintf ( 'back to itself, using Exam: %d, Series: %d \n', Baseline.Exam, Baseline.SeriesID ) );
fprintf ( fid, 'Iterate over the different spline types,\n' );
fprintf ( fid, 'stiffness, alpha parameters\n' );
fprintf ( fid, 'All cases use a width 6, Hann windowed sinc kernel\n' );
fprintf ( fid, 'All measurements are in mm\n' );

fprintf ( fid, '\n\n\n' );

fprintf ( fid, '\t\t\t\t\t\tOriginal\t\tWarped\n' );
fprintf ( fid, 'Exam\tSeries\tSpline\tStiffness\tAlpha\tMax\tMean\tRMS\tMax\tMean\tRMS\n' );



Parameters = WarpDefaults;
Parameters.InterpolationMethod = CreateHannSincKernel ( 6.0, 1000 );

for ii = 1:length ( CalibrationData )
  Calibration = ReorderSpheres ( Baseline, CalibrationData(ii) );
  disp ( sprintf ( 'Working on %d', ii ) );
  [Image, Headers] = LoadSeries ( Calibration.Directory, Calibration.Exam, Calibration.SeriesID );
  disp ( 'Loaded' );
  for ST = { 'tps', 'ebs', 'ebrs', 'vs' }
    Parameters.SplineType = char ( ST );
    
    % Alphas
    Stiffnesses = [0 logspace(-3,0,10) 5 10 20 50 100];
    Stiffnesses = [0 .5 1];
    switch Parameters.SplineType
     case 'ebs'
      Alphas = 12 * ( 1 - linspace ( 0, .5, 10 ) ) - 1;
      % Alphas = linspace ( 0, .5, 10 );
      Alphas = linspace ( .01, .5, 3 );
      Alphas = [.01 .5];
      case 'ebrs'
      Alphas = linspace ( 0, .5, 10 );
      Alphas = linspace ( 0, .5, 3 );
      Alphas = linspace ( .01, .5, 3 );
      Alphas = [.01 .5];
     otherwise
      Alphas = 0;
    end
    
    for Stiffness = Stiffnesses
      Parameters.Stiffness = Stiffness;
      
      for Alpha = Alphas
        Parameters.Alpha = Alpha;
    
        [NewImage, Result] = WarpPhantom ( Image, Calibration, Baseline, Calibration, Parameters );
        disp ( sprintf ( 'Warped in %f seconds: %s %f %f', Result.WarpTime, Parameters.SplineType, Parameters.Stiffness, Parameters.Alpha ) );
        % Detect the spheres...
        Corrected = SegmentSpheres ( Headers(1), NewImage );
        Corrected = ReorderSpheres ( Baseline, Corrected );

        CorrectedPointsRAS = [Corrected.Spheres.CenterRAS];
        CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
        BaselinePointsRAS = [Baseline.Spheres.CenterRAS];

        D = CalibrationPointsRAS - BaselinePointsRAS;
        Distance = sqrt ( sum ( D.^2 ) );
        OriginalMeanDistance = mean ( Distance );
        OriginalMaxDistance = max ( Distance );
        OriginalRMS = sqrt ( mean ( Distance.^ 2 ) );

        D = CalibrationPointsRAS - CorrectedPointsRAS;
        Distance = sqrt ( sum ( D.^2 ) );
        WarpedMeanDistance = mean ( Distance );
        WarpedMaxDistance = max ( Distance );
        WarpedRMS = sqrt ( mean ( Distance.^ 2 ) );

        fprintf ( fid, sprintf ( '%d\t%d\t%s\t%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\n', ...
                                 Calibration.Exam, ...
                                 Calibration.SeriesID, ...
                                 Parameters.SplineType, ...
                                 Parameters.Stiffness, ...
                                 Parameters.Alpha, ...
                                 OriginalMaxDistance, ...
                                 OriginalMeanDistance, ...
                                 OriginalRMS, ...
                                 WarpedMaxDistance, ...
                                 WarpedMeanDistance, ...
                                 WarpedRMS ) );
      end
    end
  end
end

fclose ( fid );












































if 0
load ReproducibilitySeries
load NonlinearSeries
load NonlinearPatient

  
% Look at the volume 6
Patient = NonlinearPatient(9);
ImageData = Patient;
Baseline = NonlinearSeries(1);
Calibration = NonlinearSeries(6);


% Warp the phantom into itself, should get spheres back where they
% came from.
% Patient = Calibration;
if ~length ( whos ( 'Image' ) )
  [Image, Headers] = LoadSeries ( Patient.Directory, Patient.Exam, Patient.SeriesID );
end

Header = Headers(1);
res = [Header.mri.pixsize Header.mri.slice_thickness];
xSpacing = (1:size(Image,1)) * res(1);
ySpacing = (1:size(Image,2)) * res(2);
zSpacing = (1:size(Image,3)) * res(3);
t = OtsuThreshold ( Image );
% Surface = isosurface ( ySpacing, xSpacing, zSpacing, Image, t, );
Surface = isosurface ( Image, t );

RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
W = TPS ( BaselinePointsRAS, CalibrationPointsRAS, 2 );
WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
VoxelToRASTForm = maketform ( 'affine', Patient.VoxelToRASTransform' );
T = [ ...
    fliptform(VoxelToRASTForm); ...
    WarpCalibrationToBaselineTForm; ...
    VoxelToRASTForm; ...
    ];
Warp = maketform ( 'composite', T );
WarpedSurface = Surface;
WarpedSurface.vertices = tforminv ( Surface.vertices, Warp );

disp ( sprintf ( 'Original volume: %0.2f', VolumeFromIsosurface ( Surface ) ) );
disp ( sprintf ( 'Warped volume: %0.2f', VolumeFromIsosurface ( WarpedSurface ) ) );
clear Surface WarpedSurface;

% View it
% Project the sphere locations into the Patient scan
OriginalFig = VolumeViewer ( Image, 'Patient scan with calibration points' );

BaselinePointsPatientVoxel = tforminv ( [Baseline.Spheres.CenterRAS]', VoxelToRASTForm );

VolumeViewer ( 'SetPoints', OriginalFig, BaselinePointsPatientVoxel );
% VolumeViewer ( 'SetBluePoints', OriginalFig, [Calibration.Spheres.COM]' );
VolumeViewer ( 'SetWarp', OriginalFig, Warp );



% Now look at warping the phantom over itself
Patient = Calibration;
[Image, Headers] = LoadSeries ( Patient.Directory, Patient.Exam, Patient.SeriesID );

% Before warp
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPoints = [Calibration.Spheres.CanonicalCOM];
BaselinePoints = [Baseline.Spheres.CanonicalCOM];
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';

DD = sqrt ( sum ( ( CalibrationPoints - BaselinePoints ).^2 ) );
disp ( 'Before the Warp: Canonical space' );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );

disp ( 'Before the Warp: RAS space' );
DD = sqrt ( sum ( ( CalibrationPointsRAS - BaselinePointsRAS ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );

% Does the warp take the Calibration.Spheres.COM into -> Baseline.Spheres.CenterRAS
VoxelToRASTForm = maketform ( 'affine', Calibration.VoxelToRASTransform' );
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = tforminv ( [Calibration.Spheres.COM]', fliptform(VoxelToRASTForm) )';
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';

disp ( 'Transform evalutation: Calibration Voxel space -> Calibration RAS' );
disp ( 'Should match previous values' );
DD = sqrt ( sum ( ( CalibrationPointsRAS - BaselinePointsRAS ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );

% Build the warp
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
W = TPS ( CalibrationPointsRAS, BaselinePointsRAS, 'vs' );
WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
T = [ ...
    fliptform(VoxelToRASTForm); ...
    WarpCalibrationToBaselineTForm; ...
    % VoxelToRASTForm; ...
    ];
Warp = maketform ( 'composite', T );

% Does the warp move the points into Baseline space
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = tforminv ( [Calibration.Spheres.COM]', Warp )';
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';

disp ( 'Warp evalutation: Calibration Voxel space -> Baseline RAS' );
disp ( 'Should be zero' );
DD = sqrt ( sum ( ( CalibrationPointsRAS - BaselinePointsRAS ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );


% Now determine how the centers of the spheres(in voxels) move
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
W = TPS ( CalibrationPointsRAS, BaselinePointsRAS, 2 );
WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
T = [ ...
    fliptform(VoxelToRASTForm); ...
    WarpCalibrationToBaselineTForm; ...
    VoxelToRASTForm; ...
    ];
Warp = maketform ( 'composite', T );
CalibrationPoints = [Calibration.Spheres.COM];
WarpedCalibrationPoints = tforminv ( CalibrationPoints', Warp )';
disp ( 'Warp evalutation: Calibration Voxel space -> Warp -> Calibration Voxel space' );
DD = sqrt ( sum ( ( WarpedCalibrationPoints - CalibrationPoints ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );



disp ( '' );
disp ( 'Warping with windowed sinc' );

frequency = 1000;
halfwidth = 6.0;
n = floor(halfwidth * frequency);
x = (halfwidth / n) * (0:n);
y = sinc ( x );
sinckernel = {halfwidth y};
alpha = 0.5;
HannWindow = alpha + (1-alpha) * cos ( 2 * pi * (0:n) / ( 2*n - 1) );
HannSincKernel = {halfwidth y.*HannWindow};

% Now do the warping, and see how things turn out
[NewImage, Result] = WarpPhantom ( Image, Patient, Baseline, Calibration, 'InterpolationMethod', HannSincKernel );

% Detect the spheres...
Corrected = SegmentSpheres ( Headers(1), NewImage );
disp ( 'Original mapping from Baseline to Calibration using ReorderSpheres' );
Foo = ReorderSpheres ( Baseline, Calibration );

disp ( 'Warped spheres from Baseline using ReorderSpheres, should be smaller' );
Corrected = ReorderSpheres ( Baseline, Corrected );

% Now we've done the warp, evaluate how far things have moved
CorrectedPoints = [Corrected.Spheres.COM];
CalibrationPoints = [Calibration.Spheres.COM];
disp ( 'Corrected spheres have moved Voxel coordinates' )
DD = sqrt ( sum ( ( CalibrationPoints - CorrectedPoints ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );

CorrectedPointsRAS = [Corrected.Spheres.CenterRAS];
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
disp ( 'Corrected spheres have moved RAS coordinates' )
DD = sqrt ( sum ( ( CalibrationPointsRAS - CorrectedPointsRAS ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );

CorrectedPointsCan = [Corrected.Spheres.CanonicalCOM];
CalibrationPointsCan = [Calibration.Spheres.CanonicalCOM];
disp ( 'Corrected spheres have moved Can coordinates' );
DD = sqrt ( sum ( ( CorrectedPointsCan - CalibrationPointsCan ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );


CorrectedPointsRAS = [Corrected.Spheres.CenterRAS];
BaselinePointsRAS = [Baseline.Spheres.CenterRAS];
disp ( 'Difference between corrected spheres and baseline RAS coordinates' )
DD = sqrt ( sum ( ( BaselinePointsRAS - CorrectedPointsRAS ).^2 ) );
disp ( sprintf ( '\tMean: %0.4f Max: %0.4f Min: %0.4f', ...
                 mean(DD), max(DD), min(DD) ) );


VoxelToRASTForm = maketform ( 'affine', Calibration.VoxelToRASTransform' );
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
BaselinePointsVoxel = tforminv ( tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm ), VoxelToRASTForm )';

% Now determine how the centers of the spheres(in voxels) move
RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
W = TPS ( BaselinePointsRAS, CalibrationPointsRAS, 2 );
WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
T = [ ...
    fliptform(VoxelToRASTForm); ...
    WarpCalibrationToBaselineTForm; ...
    VoxelToRASTForm; ...
    ];
Warp = maketform ( 'composite', T );

if ~length ( whos ( 'OriginalFig' ) )
  OriginalFig = VolumeViewer ( Image, 'Original with Baseline COM' );
end
VolumeViewer ( 'SetPoints', OriginalFig, BaselinePointsVoxel' );
VolumeViewer ( 'SetBluePoints', OriginalFig, [Calibration.Spheres.COM]' );
VolumeViewer ( 'SetWarp', OriginalFig, Warp );


if length ( whos ( 'CorrectedFig' ) )
  close ( CorrectedFig )
end
CorrectedFig = VolumeViewer ( NewImage, 'Corrected with Baseline COM' );
VolumeViewer ( 'SetPoints', CorrectedFig, BaselinePointsVoxel' );
VolumeViewer ( 'SetBluePoints', CorrectedFig, [Calibration.Spheres.COM]' );
VolumeViewer ( 'SetWarp', CorrectedFig, Warp );


% CalibrationPoints are the warped/corrected calibration phantom
% spheres
RASToCanonicalTForm = maketform ( 'affine', Spheres.RASToCanonicalTransform' );
CalibrationPoints = [Spheres.Spheres.CanonicalCOM];
BaselinePoints = [Baseline.Spheres.CanonicalCOM];
CalibrationPointsRAS = [Spheres.Spheres.CenterRAS];
BaselinePointsRAS = tforminv ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';


DD = CalibrationPoints - BaselinePoints;
DD = sqrt ( sum ( DD.^2 ) );
disp ( 'Experiment is to use Calibration -> Baseline using U(r) = r' );
disp ( sprintf ( 'Warp Calibration phantom image to baseline\nFound max displacement of %f', max(DD(:)) ) );

DD = CalibrationPointsRAS - BaselinePointsRAS;
DD = sqrt ( sum ( DD.^2 ) );
disp ( 'Experiment is to use Calibration -> Baseline using U(r) = r' );
disp ( sprintf ( 'Warp Calibration phantom image to baseline\nFound max displacement of %f', max(DD(:)) ) );

% See if we can get the transformations correct
CalibrationPoints = [Calibration.Spheres.CanonicalCOM];
BaselinePoints = [Baseline.Spheres.CanonicalCOM];

% Do deformation in RAS space
CalibrationPointsRAS = [Calibration.Spheres.CenterRAS];
BaselinePointsRAS = tformfwd ( [Baseline.Spheres.CanonicalCOM]', RASToCanonicalTForm )';
DD = CalibrationPointsRAS - BaselinePointsRAS;
DD = sqrt ( sum ( DD.^2 ) );
RASDiff = max(DD(:))

% Should match Canonical difference
DD = [Calibration.Spheres.CanonicalCOM] - [Baseline.Spheres.CanonicalCOM];
DD = sqrt ( sum ( DD.^2 ) );
CanDiff = max(DD(:))
% And it does!


% Create the warp from the CalibrationPoints to the
% BaselinePoints
% Use U(r) = r
W = TPS ( CalibrationPointsRAS, BaselinePointsRAS, 1 );
% W = TPS ( BaselinePoints, CalibrationPoints );
WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
% WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, @TPSValue, @TPSInverse, W );

VoxelToRASTForm = fliptform ( maketform ( 'affine', Calibration.VoxelToRASTransform' ) );
RASToCanonicalTForm = fliptform ( maketform ( 'affine', Calibration.RASToCanonicalTransform' ) );

RASToBaselineCanonicalTForm = maketform ( 'affine', Baseline.RASToCanonicalTransform' );
VoxelToBaselineRASTForm = maketform ( 'affine', Baseline.VoxelToRASTransform' );

T = [ fliptform(VoxelToRASTForm); ...
      fliptform(RASToCanonicalTForm); ...
      WarpCalibrationToBaselineTForm; ...
      RASToCanonicalTForm;
      VoxelToRASTForm];

% Just end up in canonical space
T = [ ...
    VoxelToRASTForm; ...
    RASToCanonicalTForm; ...
    WarpCalibrationToBaselineTForm; ...
    RASToBaselineCanonicalTForm; ...
    VoxelToBaselineRASTForm; ...
    ];
  
% However, since we want to deform real space, here we go!
T = [ ...
    VoxelToRASTForm; ...
    WarpCalibrationToBaselineTForm; ...
    fliptform(VoxelToRASTForm); ...
    ];


Warp = maketform ( 'composite', T );

CalibrationPointsVoxel = [Calibration.Spheres.COM];
BaselinePointsVoxel = [Baseline.Spheres.COM];

Warped = tforminv ( CalibrationPointsVoxel', Warp)';
WarpedRAS = tforminv ( Warped', VoxelToBaselineRASTForm );

% Warped = CalibrationPointsVoxel;

DD = Warped - CalibrationPointsVoxel;
DD = Warped - BaselinePointsVoxel;
DD = sqrt ( sum ( DD.^2 ) );
max(DD(:))
min(DD(:))




% I expect that Calibration.VoxelToRASTransform * [0 0 0 1]' ==
% tforminv ( [0 0 0], VoxelToRASTForm )
Pt = Calibration.Spheres(1).COM
PtRAS = Calibration.Spheres(1).CenterRAS
PtCan = Calibration.Spheres(1).CanonicalCOM

Calibration.VoxelToRASTransform * [Pt; 1]
tforminv ( Pt, VoxelToRASTForm )
PtRAS

Calibration.RASToCanonicalTransform * Calibration.VoxelToRASTransform * [Pt; 1]
f = tforminv ( tforminv ( Pt, VoxelToRASTForm ), RASToCanonicalTForm )
tforminv ( Pt, Warp )
PtCan

DD = tforminv ( [Calibration.Spheres.COM]', Warp )' - [Calibration.Spheres.CanonicalCOM];
DD = sqrt ( sum ( DD.^2 ) );
max(DD(:))






% Old junk from here down

[Image, Headers] = LoadSeries ( Patient.Directory, Patient.Exam, Patient.SeriesID );
fn = [Patient.Directory, filesep, 'TPSWarp', int2str(Patient.SeriesID), '.mat'];
S = load ( fn );
NewImage = S.NewImage;
clear S;



[Image, Headers] = LoadSeries ( Patient.Directory, Patient.Exam, Patient.SeriesID );




[NewImage, Results] = WarpPhantom ( Image, Patient, Baseline, Calibration, 50 );

  CalibrationPoints = [Calibration.Spheres.CanonicalCOM];
  BaselinePoints = [Baseline.Spheres.CanonicalCOM];

  % Create the warp from the CalibrationPoints to the BaselinePoints
  W = TPS ( CalibrationPoints, BaselinePoints, 2);
  WR2LogR2 = TPS ( CalibrationPoints, BaselinePoints, 1);
  W2 = TPS ( BaselinePoints, CalibrationPoints, 2 );

  if 0,
  % Maximum distance before...
  DD = CalibrationPoints - BaselinePoints;
  DD = sqrt ( sum ( DD.^2 ) );
  max(DD(:))  
  % Maximum distance after...
  DD = TPSValue ( CalibrationPoints', W )' - BaselinePoints;
  DD = sqrt ( sum ( DD.^2 ) );
  max ( DD(:) )

  % Forward and back
  DD = TPSValue ( TPSValue ( BaselinePoints', W2 ), W )' - BaselinePoints;
  DD = sqrt ( sum ( DD.^2 ) );
  max ( DD(:) )
  end
  
  WarpCalibrationToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );
  

  
  
  VoxelToRASTForm = maketform ( 'affine', ImageData.VoxelToRASTransform' );
  RASToCanonicalTForm = maketform ( 'affine', Calibration.RASToCanonicalTransform' );
  
  
  clear T;
  T = [ fliptform(VoxelToRASTForm); ...
        fliptform(RASToCanonicalTForm); ...
        WarpCalibrationToBaselineTForm; ...
        RASToCanonicalTForm;
        VoxelToRASTForm];
  
  Warp = maketform ( 'composite', T );

  SliceNumber = 50;
  Slice = Image(:,:,SliceNumber);
  
  TranslateTForm = maketform ( 'affine', [1 0 0 0; 0 1 0 0; 0 0 1 -SliceNumber; 0 0 0 1]' );
  WarpSlice = maketform ( 'composite', [TranslateTForm; T] );
  

  R = makeresampler ( {'cubic', 'cubic', 'cubic'}, 'fill' );
  R = makeresampler ( {'nearest', 'nearest', 'nearest'}, 'fill' );
  frequency = 1000;
  halfwidth = 2.0;
  n = floor(halfwidth * frequency);
  x = (halfwidth / n) * (0:n);
  y = sinc ( x );
  sinckernel = {halfwidth y};
  R = makeresampler ( sinckernel, 'fill' );
  
  % B = TFORMARRAY(A,T,R,   TDIMS_A,  TDIMS_B,  TSIZE_B,   TMAP_B,F)
  NewSlice = tformarray ( Image, WarpSlice, R, [1 2 3], [1 2 3], [256 256 1], [], 0 );
  
  figure ( 2 ); colormap ( gray ); clf
  imagesc ( NewSlice )
  
  [PixelCount, Area] = SliceArea ( Slice, OtsuThreshold(Slice) )
  [NewPixelCount, NewArea] = SliceArea ( NewSlice, OtsuThreshold(NewSlice) )
  
  
  % Create a contour
  t = OtsuThreshold ( Image );
  C = GetContour ( Slice, t );
  C = [C, 50*ones(size(C,1),1)];
  
  CWarp = tforminv ( C, Warp );
  
  D = sum ( (C' - CWarp').^2 );
  
  figure ( 1 ); clf; colormap ( gray );
  imagesc ( Slice );
  hold on
  plot ( C(:,1), C(:,2), 'b+' );
  plot ( CWarp(:,1), C(:,2), 'ro' );
  legend ( 'Original', 'Corrected' )
  
figure ( 1 ); clf;

BaselinePoints = [Baseline.Spheres.CanonicalCOM];
CalibrationPoints = [Calibration.Spheres.CanonicalCOM];

% make a grid
[xx,yy,zz] = meshgrid ( 1:10:256, 1:10:256, 50 );

Pts = [xx(:) yy(:) zz(:)];

% A = FitSpheres ( BaselinePoints', CalibrationPoints' );
% PolynomialWarpToBaselineTForm = maketform ( 'custom', 3, 3, [], @PolyFitFunction, A );

WCalibrationToBaseline = TPS ( CalibrationPoints, BaselinePoints );
WBaselineToCalibration = TPS ( BaselinePoints, CalibrationPoints );

V = random ( 'Uniform', -10, 300, 200, 3 );
V = Pts;
P1 = TPSValue ( V, WCalibrationToBaseline );
P2 = TPSValue ( P1, WBaselineToCalibration );
D = sqrt ( sum ( ( P2' - V' ).^2 ) );
max ( D(:) )

P1 = TPSValue ( V, WCalibrationToBaseline );
P2 = TPSInverse ( P1, WCalibrationToBaseline );
D = sqrt ( sum ( ( P2' - V' ).^2 ) );
max ( D(:) )
                                                         

P1 = TPSValue ( V, WBaselineToCalibration );
P2 = TPSValue ( P1, WCalibrationToBaseline );
D = sqrt ( sum ( ( P2' - V' ).^2 ) );
D1 = sqrt ( sum ( ( P1' - V' ).^2 ) );
max ( D(:) )
max ( D1(:) )



TPSWarpToBaselineTForm = maketform ( 'custom', 3, 3, [], @TPSValue, W );

VoxelToRASTForm = maketform ( 'affine', NonlinearPatient6.VoxelToRASTransform' );
RASToCanonicalTForm = maketform ( 'affine', NonlinearSeries3.RASToCanonicalTransform' );

clear T;
T(1) = fliptform ( VoxelToRASTForm );
T(2) = fliptform ( RASToCanonicalTForm );

T(3) = TPSWarpToBaselineTForm;
% T(3) = PolynomialWarpToBaselineTForm;

T(4) = RASToCanonicalTForm;
T(5) = VoxelToRASTForm;

Warp = maketform ( 'composite', T );
R = makeresampler ( 'nearest', 'fill' );

Pts2 = tforminv ( Pts, Warp );

figure ( 1 ); clf; hold off
imagesc ( Image(:,:,50) -NewImage(:,:,50) );
colormap ( gray )
hold on
quiver ( xx(:), yy(:), Pts2(:,1) - xx(:), Pts2(:,2) - yy(:), 0 )

% Check the calibration phantom points
PP = tforminv ( CalibrationPoints', Warp );
Lx = CalibrationPoints(1,:)';
Ly = CalibrationPoints(2,:)';
Lz = CalibrationPoints(3,:)';
Sx = BaselinePoints(1,:)';
Sy = BaselinePoints(2,:)';
Sz = BaselinePoints(3,:)';
Px = PP(:,1);
Py = PP(:,2);
Pz = PP(:,3);

figure ( 2 ); clf; hold off
plot3 ( Lx, Ly, Lz, '+r' );
hold on
quiver3 ( Lx, Ly, Lz, Sx - Lx, Sy - Ly, Sz - Lz, 0 );
% quiver3 ( Lx, Ly, Lz, Px - Lx, Py - Ly, Pz - Lz );
plot3 ( Sx, Sy, Sz, '+g' );


I1 = Image(:,:,50 );
I2 = NewImage (:,:,50);
t1 = OtsuThreshold ( I1 );
t2 = OtsuThreshold ( I2 );

[P,A,C1] = SliceArea ( I1, t1, 1 );
[P,A,C2] = SliceArea ( I2, t2, 1 );

figure ( 3 ); clf; hold off;
imagesc ( I1 ); colormap ( gray );
hold on
plot ( C1(1,:), C1(2,:), '+r' );
plot ( C2(1,:), C2(2,:), '+g' );



% Forward / Inverse check




Pt = CalibrationPoints(:,1);
x = fminunc ( @Foo, Pt', [], W, Pt' );


end