function ISpheres = FindInverseSphereLocations ( Spheres, Results, PrintStatus )
% ISpheres = FindInverseSphereLocations ( Spheres, Results, [PrintStatus] )
%
% Calculate and return where Spheres would warp to given the
% transformation in Results(returned by WarpPhantom).  Use a simple
% gradient descent.  The optional PrintStatus indicates that a
% printing of status messages is wanted.  Default is PrintStatus = 0.
  

  if ~exist ( 'PrintStatus', 'var' )
    PrintStatus = 0;
  end

  ISpheres = Spheres;
  
  VoxelToRASTForm = maketform ( 'affine', ISpheres.VoxelToRASTransform' );
  RASToCanonicalTForm = maketform ( 'affine', ISpheres.RASToCanonicalTransform' );
  
  for ID = 1:length(ISpheres.Spheres)
    
    [P, Delta] = TPSInverseInternal ( ISpheres.Spheres(ID).VoxelCOM, Results.Warp, ISpheres.Spheres(ID).VoxelCOM );
    ISpheres.Spheres(ID).RASCOM = tformfwd ( P, VoxelToRASTForm )';
    ISpheres.Spheres(ID).CanonicalCOM = tformfwd ( ISpheres.Spheres(ID).RASCOM', RASToCanonicalTransform )';
    ISpheres.Spheres(ID).VoxelCOM = P;
    
    if PrintStatus
      disp ( sprintf ( 'Sphere: %d', ID ) );
      disp ( sprintf ( '\tFound: (%0.2f, %0.2f, %0.2f)', P(1), P(2), P(3) ) );
    
      PPrime = tforminv ( P, Results.Warp );
      
      disp ( sprintf ( '\tWarped to: (%0.2f, %0.2f, %0.2f)', PPrime(1), PPrime(2), PPrime(3) ) );
      disp ( sprintf ( '\tShould be: (%0.2f, %0.2f, %0.2f)', Spheres.Spheres(ID).VoxelCOM(1), Spheres.Spheres(ID).VoxelCOM(2), Spheres.Spheres(ID).VoxelCOM(3) ) );
    end
    
  end
  
  
  
  
  
function [P, Delta] = TPSInverseInternal ( X, W, Target )
% Use a very simple gradient descent
  P = X;
  Delta = Target - tforminv ( P, W )';
  Iteration = 0;
  while norm ( Delta ) > 0.005
    Delta = Target - tforminv ( P, W )';
    P = P + Delta./2.0;
    if Iteration > 100
      error ( 'Failed to converge' )
    end
    Iteration = Iteration + 1;
  end
    
