function [I, Headers] = LoadDICOMSeries ( Pattern, CustomDictionary, SwapBytes, FileList , DataType)

if ~exist('DataType')
        DataType='double';
end
if ~exist ( 'CustomDictionary' )
    CustomDictionary = 0;
end

if (exist ( 'SwapBytes' ) ~= 1)
    SwapBytes = 0;
end

Dictionary = 'dicom-dict.mat';
if CustomDictionary
    % Do we have a custom directory present?
    Dictionary = which ( 'local-dicom-dict.txt' );
    if length ( Dictionary ) ~= 0
        Dictionary = 'local-dicom-dict.txt';
    end
end

% Have we already warned of byteswapping?
WarnedOfByteSwapping = 0;

% Find the files
if ~exist('FileList','var')
    FileList = dir ( Pattern );
    [Directory, name, ext, version] = fileparts ( Pattern );
elseif ~exist(Pattern,'dir')
    [Directory, name, ext, version] = fileparts ( Pattern );
elseif exist(Pattern,'dir')
    Directory = Pattern;
end
Count = length ( FileList );


Directory = [Directory filesep];
%save
% Read the first one to get the size
Image = dicomread ( [Directory FileList(1).name] );

if CustomDictionary
    Header = dicominfo ( [Directory FileList(1).name], 'dictionary', Dictionary );
else
    Header = dicominfo ( [Directory FileList(1).name] );
end
Headers = Header;
[r, c] = size ( Image );
I = zeros ( r, c, Count, DataType );
%I = zeros ( r, c, Count, 'int16' );

for i = 1:Count,
    FN = [Directory FileList(i).name];
    Image = dicomread ( FN );
    if length ( Image(:) < 0 ) & SwapBytes
        if ~WarnedOfByteSwapping
            warning 'Byte swap detected';
            WarnedOfByteSwapping = 1;
        end
        Image = ByteSwap16 ( Image );
    end
    if CustomDictionary
        Header = dicominfo ( FN, 'dictionary', Dictionary );
    else
        Header = dicominfo ( FN );
    end
    I(:,:,i) = Image;
    d = setdiff ( fieldnames ( Header ), fieldnames ( Headers(1) ) );
    if length ( d )
        % Remove offending fields!
        for jj = 1:length ( d )
            % Headers(1).TriggerTime = Header.TriggerTime;
            Header = rmfield ( Header, d(jj) );
        end
        % d = setdiff ( fieldnames ( Header ), fieldnames ( Headers(1) ) )
    end
    Headers(i) = Header;
end

% Re-ordering step, want to order by biggest slice location to
% smallest, then by echo time...
if isfield ( Headers(1), 'EchoTime' )
    s = [Headers.InstanceNumber; Headers.EchoTime]';
else
    s = [Headers.SliceLocation; Headers.SliceLocation]';
end
% s(:,1) = -1 * s(:,1);
[Foo, Idx] = sortrows ( s, [1 2] );
I(:,:,:) = I(:,:,Idx);
Headers = Headers(Idx);

% Make sure we are consistent!
XYZ = CoordinateSystems ( Headers, 'XYZ' );
IPP = [Headers.ImagePositionPatient]';
Len = length ( Headers );
o = ones ( Len, 1 );
t = 1:Len;
t = t';
Locs = XYZ * [o o t o]';
Locs = Locs';
Locs = Locs(:,1:3);
DD = Locs - IPP;
DD = sqrt ( sum ( DD.^2, 2 ) );
 if max(DD) > 0.1
   % Reverse
   disp ( 'flipping' );
   Idx = Len:-1:1;
   I(:,:,:) = I(:,:,Idx);
   Headers = Headers(Idx);
 end
  
  


%  save
return
% [Image, Headers] = LoadDicomSeries (
% '/home/blezek/src/MION/ADData/2003-01-27-DualEcho3T/SE1/*.dcm' );

