function [PixelCount, Area, Contour] = SliceArea ( Slice, t, varargin )
% Calculate the area of the slice, given a threshold
% SliceArea ( Slice, T, Display, BorderSize, Filter )
% Slice is the image to process
% T is the threshold used to localize edges
% Display is a boolean to indicate if the results should be shown
% BorderSize is the width in pixels of the area around the object
% to use when finding edges
% Filter is the filtereded LOG for this slice

Display = 0;
if nargin >= 3
  % Third arg is display, force boolean
  Display = 1 & varargin{1};
end

BorderRadius = 13;
if nargin >= 4
  BorderRadius = varargin{2};
end

Filter = [];
if nargin >= 5
  Filter = varargin{3};
end


if Filter == []
  G = fspecial ( 'gaussian', 7, 1.5 );
  H = fspecial ( 'log', 7, 1.5 );
  L = imfilter ( imfilter ( Slice, G ), H );
  % [dx, dy] = gradient ( Slice );
  % Angle = atan2 ( dy, dx );
else
  L = Filter;
end

% Border is the outer single layer of pixels
Mask = Slice >= t;
PixelCount = length ( find ( Mask ) );
Area = 0;
Contour = [];
% Exit early if we can
if PixelCount == 0
  return
end

st = strel ( [1 1 1; 1 1 1; 1 1 1] );
LargeStrel = strel ( 'square', BorderRadius );

Border = bwmorph ( Mask & ~imerode ( Mask, st ), 'skel', Inf );
InteriorMask = imerode ( Mask, st );
ExteriorMask = imdilate ( Mask, LargeStrel );
ThickBorder = ExteriorMask & ~InteriorMask;
MaskedLaplacian = L .* double ( ThickBorder );

% Fill in the center of the masked laplacian
LMin = min ( MaskedLaplacian(:) );
LMax = max ( MaskedLaplacian(:) );
Interior = LMin .* double ( InteriorMask );
Exterior = LMax .* double ( ~ExteriorMask );
MaskedLaplacian = MaskedLaplacian + Interior + Exterior;


C = contour ( MaskedLaplacian, [0.0 0.0] );
% Find the largest contour
Length = size ( C, 2 );
NextIndex = 1;
ContourIndex = [];
while NextIndex < Length
  ContourSize = C(2,NextIndex);
  ContourIndex = [ContourIndex [NextIndex; ContourSize]];
  NextIndex = NextIndex + ContourSize + 1;
end

if ContourIndex == []
  % figure; colormap ( gray ); imagesc ( MaskedLaplacian )
  return;
end

Y = sortrows ( ContourIndex', [2] );
LongestContourIndex = Y(end,1);
LongestContourSize = Y(end,2);
StartIdx = LongestContourIndex + 1;
EndIdx = StartIdx + LongestContourSize - 1;
Idx = StartIdx:EndIdx;
Area = polyarea ( C(1,Idx), C(2,Idx) );
Contour = C(1:2,Idx);

if Display
  [rr, cc] = find ( Border );
  figure
  clf
  RGB = MaskImage ( Slice, Mask );
  RGB = MaskImage ( RGB, Border, 0.9 );
  RGB = MaskImage ( RGB, ~ExteriorMask, 0.4, [0 1 0] );
  imagesc ( RGB );
  hold on
  plot ( cc, rr, 'b+' )
  contour ( MaskedLaplacian, [0.0 0.0], 'y-' );
end

return




OriginalBorderArea
BorderArea
MaskedPixelCount = length ( find ( Mask ) )

% For each border pixel, find the zero crossing of the laplacian
[rr, cc] = find ( Border );

% Line length, and sampling
LineLength = 5;
LineSampling = LineLength * 10;

NewBorderX = [];
NewBorderY = [];


for BorderIndex = 1:length(rr)
  s = cc(BorderIndex);
  t = rr(BorderIndex);
  a = Angle(t,s);

  px = [ s+cos(a)*LineLength/2, s-cos(a)*LineLength/2];
  py = [ t+sin(a)*LineLength/2, t-sin(a)*LineLength/2];

  [cx, cy, v] = improfile ( MaskedLaplacian, px, py, LineSampling, 'bicubic' );
  
  % find the smallest non-zero entry
  idx = find ( v >= 0.0 );
  idx = find ( v == min ( v(idx) ) );
  NewBorderX = [NewBorderX cx(idx)];
  NewBorderY = [NewBorderY cy(idx)];

end

[K, BorderArea] = convhull ( NewBorderX, NewBorderY );
[OriginalK, OriginalBorderArea] = convhull ( cc, rr );

figure
clf
imagesc ( MaskImage ( Slice, Border, 0.9 ) )
hold on
plot ( cc, rr, 'b+' )
plot ( NewBorderX(K), NewBorderY(K), 'r-', NewBorderX, NewBorderY, 'g+' )
plot ( NewBorderX, NewBorderY, 'g+' )
contour ( MaskedLaplacian, [0.0 0.0], 'y-' );

C = contour ( MaskedLaplacian, [0.0 0.0] );


OriginalBorderArea
BorderArea
MaskedPixelCount = length ( find ( Mask ) )
ContourArea = polyarea ( C(1,3:end), C(2,3:end) )


  