function metric=SphereDeltaHistogram(FileName); %input is the filename of Dan's output, a .mat file.

Temp=load(FileName);

RawDat=Temp.RawSpheres;
CorrDat=Temp.CorrectedSpheres;

%reformat data to an array with as many rows as there are sphere pairs, and
%three columns. Each column represents a scanner.

NumSpheres=length(RawDat(1).Spheres);  %Detects number of spheres

RawDistances=zeros(NumSpheres,3); %allocate arrays
CorrDistances=zeros(NumSpheres,3);

rowind=0;
for ind1=1:NumSpheres-1         %loop through all spheres
    for ind2=ind1+1:NumSpheres  %loop through all spheres greater than ind1 sphere
        rowind=rowind+1;        %row index for final output
        for colind=1:3
            RawDistances(rowind,colind)=RawDat(colind).D(ind1,ind2);
            CorrDistances(rowind,colind)=CorrDat(colind).D(ind1,ind2);
        end
    end
end
%rowind now contains the total number of rows.
%Next go to computing histograms for raw and corrected data

%Set up array with 'spreads' - maximum distance measurement minus minimum
%distance measurement, among the three scanners.
RawSpread=max(RawDistances,[],2)-min(RawDistances,[],2);    
CorrSpread=max(CorrDistances,[],2)-min(CorrDistances,[],2);

%Compute histogram on the spreads

binsize=.05; %bin size for the histogram
numbins=ceil(max(RawSpread)/binsize);

Hist=zeros(numbins,3);  %preallocate array for histogram. first column is bin lower bound, 
                        %Second column is raw data histogram, 3rd column is
                        %corrected data histogram
for ind=1:numbins
    Hist(ind,1)=(ind-1)*binsize;
    Hist(ind,2)=sum((RawSpread>=((ind-1)*binsize)).*(RawSpread<(ind*binsize)));%counts number of distances in this bin
    Hist(ind,3)=sum((CorrSpread>=((ind-1)*binsize)).*(CorrSpread<(ind*binsize)));
end

%plot histograms, if you like....there is a 4-point filter for aesthetics.
b=[.25,.25,.25,.25];
y1=filter(b,1,Hist(:,2));
y2=filter(b,1,Hist(:,3));
plot(Hist(:,1),y1);
hold on;
plot(Hist(:,1),y2,'r');
legend('raw','corrected');

Hist(:,1).*Hist(:,2);
meanraw=sum(Hist(:,1).*Hist(:,2))/sum(Hist(:,2));
meancorr=sum(Hist(:,1).*Hist(:,3))/sum(Hist(:,3));

disp('raw and corrected histogram mean values:')
metric=[meanraw,meancorr]   %metric contains the average values of the raw and corrected 
                            %histograms, respectively





    


